from tkinter import *


row = 0
col = 0
__root = Tk()
__root.overrideredirect(True)


e = Entry(__root,  font=(None, 48))

e.config(width=15)
e.grid(row=row, column=col, padx=0, pady=0)
__root.geometry('+%d+%d' % (100, 100))
while True:
    __root.update_idletasks()
    pass
    __root.update()



from tkinter import *
import multiprocessing as mp
import multiprocessing as q

class LEDMatrix:

    def __enter__(self):
        return self
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def __del__(self):
        self.continue_proc = False
        if self.q is not None:
            self.q.close()
    def __init__(self, rows = 8, cols = 8, color = (0,0,0), origin_x=5, origin_y=5,ledWidth =2, ledHeight=1, ledPadx=2,ledPady=2):

        self.origin_x = origin_x
        self.origin_y = origin_y
        self.__root = None
        self.continue_proc = True
        self.__rows = rows
        self.__cols = cols
        self.__ledWidth =ledWidth
        self.q = q.Queue()
        self.__ledHeight = ledHeight
        self.__ledPadx = ledPadx
        self.__ledPady = ledPady
        self.__p = None
        self.__ledColor = self.__from_rgb(color)
        self.__matrix = [[0 for x in range(cols)] for y in range(rows)]
        self.__show()

    def __from_rgb(self, rgb):

        if type(rgb) is not tuple:
            if type(rgb) is int:
                rgb = rgb, 0, 0

        return "#%02x%02x%02x" % rgb

    def __show(self):

        self.__p = mp.Process(target=self._create_matrix, args=(self.q,))

        self.__p.start()

    def _create_matrix(self, q):
        row = 0
        col = 0
        self.__root = Tk()
        self.__root.overrideredirect(True)
        #self.__root.title = "LED Matrix"
        new_row = None

        def on_closing():
            self.continue_proc=False

        self.__root.protocol("WM_DELETE_WINDOW", on_closing)

        for color in range(self.__rows*self.__cols):

            e = Label(self.__root, text="", background=self.__ledColor,font=(None, -10))
            e.config(height=self.__ledHeight, width=self.__ledWidth)
            e.grid(row=row, column=col, padx=self.__ledPadx,pady=self.__ledPady)
            self.__matrix[row][col]=e
            #e.pack(expand=YES, fill=BOTH)
            col += 1
            if (col == self.__cols):
                row+=1
                col = 0

        self.__root.geometry('+%d+%d' % (self.origin_x, self.origin_y))
        while self.continue_proc:
            self.__root.update_idletasks()
            if q.qsize() > 0:
                action, *params = q.get(block=True)
                # Do Work
                if action == "set_pixel":
                    self.__matrix[params[1]][params[0]].config(background=self.__from_rgb(params[2]))
                elif action=="set_pixels":
                    matrix = params[0]

                    for r, row in enumerate(matrix):
                        for c, column in enumerate(row):

                            self.__matrix[r][c].config(background=self.__from_rgb(column))

                elif action=="set_pixels_color":
                    matrix = params[0]
                    color = params[1]
                    for r, row in enumerate(matrix):
                        for c, column in enumerate(row):
                            if bool(column):
                                self.__matrix[r][c].config(background=self.__from_rgb(color))
                elif action=="clear":
                    self.__clear()
                elif action=="close":
                    self.continue_proc = False

                    self.q.close()
                    self.__root.destroy()
                    break

            self.__root.update()
    def set_pixels_color(self, matrix, color):

        self.q.put(("set_pixels_color",matrix, color))

    def set_pixels(self, matrix, color):

        self.q.put(("set_pixels",matrix))


    def set_pixel(self, x, y, color):

        self.q.put(("set_pixel",x,y,color))

    def close(self):
        self.continue_proc = False
        if self.q is not None:
            self.q.put(("close",[]))

    def clear(self):

        self.q.put(("clear",(0,)))

    def __clear(self):
        for row in range(self.__rows):
            for col in range(self.__cols):
                self.__matrix[row][col].config(background=self.__ledColor)
                #self.__matrix[row][col].config(background=self.__ledColor)

if __name__ == "__main__":
    import time

    matrix = [
        [255, 0, 0, 0, 0, 0, 0, 1],
        [1, 0, 0, 0, 0, 0, 0, 1],
        [1, 0, 0, 0, 0, 0, 0, 1],
        [1, 1, 1, 1, 1, 1, 1, 1],
        [1, 1, 1, 1, 1, 1, 1, 1],
        [1, 0, 0, 0, 0, 0, 0, 1],
        [1, 0, 0, 0, 0, 0, 0, 1],
        [1, 0, 0, 0, 0, 0, 0, (255,255,0)]
    ]

    with LEDMatrix(cols=8, rows=8) as b:

        b.set_pixels(matrix, (255,0,0))
        time.sleep(5)



"""

"""
import time
from pihat_widget import image

class TrafficLight(image.ImageLoader):

    def __init__(self, origin_x=5, origin_y=5, origin_width=140, origin_height=330, offsetx=10, offsety=10):
        super().__init__(origin_x, origin_y, origin_width, origin_height, offsetx, offsety)
        self.set_state("000")

    """
    Displays simulated Traffic signal
    """
    def set_state(self, state):
        """
        Set the state using a binary string
        :param state: binary string controlling red, amber green. 100 = (red(on) amber(off) green(off))
        :return: Exception on red, green and green amber
        """
        if state == "100":
            status = "red"
        elif state == "110":
            status ="redamber"
        elif state == "010":
            status = "amber"
        elif state == "001":
            status = "green"
        elif state == "000":
            status = "off"
        else:
            raise ValueError(f"Invalid Light State {state}")
        s = int(state, 2)
        self.q.put(("load",status))

if __name__ == "__main__":

    lg = TrafficLight()
    count =0
    while count<1:
##
        time.sleep(5)
        lg.set_state("110")
        time.sleep(2.7)
        lg.set_state("001")
        time.sleep(5)
        lg.set_state("010")
        time.sleep(2.7)
        count+=1



"""
Contains classes for
    LEDCharacter        - Single character number display
    LEFMultiCharacter   - Multicharacter display
"""
from pihat_widget import image
import time

class LEDCharacter(image.ImageLoader):
    """
    Display as single numeric character
    """
    def __init__(self, x, y):
        super().__init__(origin_x=x, origin_y=y, origin_width=45, origin_height=77, offsetx=0, offsety=0)
        self.__number = None

    @property
    def Number(self):
        """
        Returns the current displayed number
        :return: Displayed number
        """
        return self.__number
    @Number.setter
    def Number(self, number):
        """
        Allows the displayed value to be set
        :param number: Number to be set
        :return: None
        """
        if type(number) is int:
            self.q.put(("load",str(number)))
            self.__number = number
        else:
            raise ValueError("Number must be an int")


class LEDMultiCharacter:
    """
    Displays a multi character numeric character
    """
    def __del__(self):
        self.close()

    def __init__(self, digits = 2, x = 10, y = 10):
        """
        Initialises the class
        :param digits: Number of numeric digits to display
        """
        self.__digits = digits
        self.__chars = [LEDCharacter(x+(i*45),y) for i in range(digits)]
        self.__number = None
        for z in self.__chars:
            z.Number =0

    @property
    def Number(self):
        return self.__number

    @Number.setter
    def Number(self, number):
        if type(number) is int:

            num = f"{number:0{self.__digits}d}"
            for c,n in enumerate(num):
                self.__chars[c].Number = int(n)

        else:
            raise ValueError("Number must be an int")
    def close(self):
        for c in self.__chars:
            c.close()

if __name__ == "__main__":

    c = LEDCharacter(x = 150, y = 150)
    for i in range(10):
        c.Number = i
        time.sleep(0.5)
    time.sleep(3)
    c.close()

    c = LEDMultiCharacter(digits = 3, y=120)
    for i in range(105):
        c.Number =i
    c.close()

